﻿/**
 * Behavior
 *
 * $Project: CLX.WBT-Tracker$
 * $Date: 12.07.2011 17:12:36$ 
 * $Revision: 1$
 *
 * @package    	clx.scorm
 * @copyright  	Copyright (c) Crealogix AG
 * @link		www.crealogix.com
 * @since       1.1.0
 * @author      Andreas Häberli <andreas.haeberli@crealogix.com>
 * @version		$Id: Behavior.js,v 1.0, 2011-07-12 15:12:36Z, Andreas Haeberli$
 */

dojo.require('clx.scorm.Data');
dojo.require('clx.scorm.Format');

dojo.provide('clx.scorm.Behavior');
dojo.declare('clx.scorm.Behavior',null,{
	
	/*
	 * constructor
	 * 
	 * @param object storage
	 */		
	constructor: function(storage) {
		this._storage = storage;
		this._data = new clx.scorm.Data(this._storage);
		this._isCredit = false;
		this._totalTime = null;
		this._isInitialize = false;		
		
		if(!this._storage.isSynch()) {
			dojo.connect(this._storage, 'onAfterRead', this, 'onInitializeData');
			this.initializeData();
		}
	},
		
	/*
	 * initializeData
	 * 
	 * @return string
	 */		
	initializeData:function() {
		if(this._storage.isSynch()) {
			var result = this._storage.read();
			this.afterInitializeData();		
			return result;
		} else {
			// TODO: Error handling
			if(!this._isInitialize) {
				this._storage.read();
			}
			return true;
		}
	},
	
	/*
	 * onInitializeData
	 * 
	 * @return void
	 */		
	onInitializeData:function() {
		// TODO: Error handling
		this.afterInitializeData();
		this._isInitialize = true;
	},
	
	/*
	 * initializeData
	 * 
	 * @return string
	 */		
	afterInitializeData:function() {
		this._isCredit = (this._data.get('cmi.core.credit') == 'credit') ? true:false;
		this._totalTime = new clx.scorm.Format().timeStringToNum(this._data.get('cmi.core.total_time', true)); 
	},
	
	/*
	 * commitData
	 * 
	 * @return string
	 */		
	commitData:function() {
		return this._storage.commit(); 		
	},
	
	/*
	 * finish
	 * 
	 * @return string
	 */		
	finish:function() {
		this._setCmiCoreEntry();
		this._setCmiCoreLessonStatus();
		return this._storage.finish(); 
	},
	
	/*
	 * getData
	 * 
	 * @param string parameter
	 * @return mixed string|clx.scorm.Error
	 */		
	getData:function(parameter) {
		return this._data.get(parameter);
	},
	
	/*
	 * setData
	 * 
	 * @param string parameter
	 * @param string value
	 * @return mixed string|clx.scorm.Error
	 */		
	setData:function(parameter, value) {
		var result = this._data.set(parameter, value);
		if(typeof(result) == 'string') {
			switch (parameter) {
				case 'cmi.core.session_time':
					this._setCmiCoreSessionTime();
					break;
				case 'cmi.core.exit':
					this._setCmiCoreExit();
					break;
				default:
					if(result != 'false') {
						this._setItemToDataCommitObject(parameter, value);
					}
				    break;
			}
		}
		return result;
	},
	
	/*
	 * _setCmiCoreSessionTime
	 * A SCO is able, in a single running, to perform multiple sets of the cmi.core.session_time.
	 * When the SCO issues the LMSFinish() or the user navigates away,
	 * the LMS should take the last cmi.core.session_time that the SCO set (if there was a set)
	 * and accumulate this time to the cmi.core.total_time. 
	 * Upon subsequent launch of the SCO, and a LMSGetValue() call for cmi.core.total_time,
	 * the LMS should return the accumulated time.
	 * LMS’s should not accumulate multiple time sent to the LMS by the LMSSetValue() call for cmi.core.session_time.
	 * If multiple calls to LMSSetValue() for cmi.core.session_time are made
	 * the LMS should overwrite any existing value that it is persisting for cmi.core.session_time.
	 * 
	 */		
	_setCmiCoreSessionTime:function() {
		var sessionTime = this._data.get('cmi.core.session_time', true);
		var f = new clx.scorm.Format();
		totalTime = f.numToTimeString(f.timeStringToNum(sessionTime) + this._totalTime);
		this._data.set('cmi.core.total_time', totalTime, true);
		this._setItemToDataFinishObject('cmi.core.session_time', sessionTime);
		this._setItemToDataFinishObject('cmi.core.total_time', totalTime);
	},
	
	/*
	 * _setCmiCoreExit
	 * A SCO has the option of setting the cmi.core.exit to one of four values.
	 * Based on the value, the LMS behavior should be as follows:
	 * - If the SCO set the cmi.core.exit to "time-out" the LMS should set cmi.core.entry to "" (empty)
	 *   upon the next launching of the SCO.
	 * - If the SCO set the cmi.core.exit to "suspend" the LMS should set the cmi.core.entry to "resume"
	 *   upon the next launching of the SCO.
	 * - If the SCO set the cmi.core.exit to "logout" the LMS should set the cmi.core.entry to "" (empty) 
	 *   upon the next launching of the SCO.
	 *   In addition, the LMS should log the student out of the course when the SCO that set the cmi.core.exit to "logout"
	 *   has issued the LMSFinish() or the user navigates away.
	 * - If the SCO set the cmi.core.exit to "" (empty) the LMS should set the cmi.core.entry to "" (empty)
	 *   upon the next launching of the SCO.
	 * - If the SCO did not set the cmi.core.exit to any value that LMS should set cmi.core.entry to "" (empty)
	 *   upon the next launching of the SCO.
	 * 
	 */		
	_setCmiCoreExit:function() {
		var value = this._data.get('cmi.core.exit', true);
		this._setItemToDataFinishObject('cmi.core.exit', value);
		this._setCmiCoreEntry();
	},
	
	/*
	 * _setCmiCoreEntry
	 * Upon receiving an LMSFinish() or the user navigates away, the LMS should set the cmi.core.entry to either "" – (empty) or "resume".
	 * This is determined by the LMS looking at the value that the SCO had set for the cmi.core.exit.
	 * If the SCO set the value of cmi.core.exit to "suspend", then the LMS will set cmi.core.entry to "resume" upon the next launch of the SCO.
	 * If the SCO set cmi.core.exit to a value other than "resume" or did not set the value at all, the LMS will set cmi.core.entry to "" (empty).
	 * 
	 */		
	_setCmiCoreEntry:function() {
		var value = (this._data.get('cmi.core.exit', true) == 'suspend') ? 'resume':'';
		var result = this._data.set('cmi.core.entry', value, true);
		if(result != 'false') {
			this._setItemToDataFinishObject('cmi.core.entry', value);
		}
	},
	
	/*
	 * _setCmiCoreLessonStatus
	 * If a SCO sets the cmi.core.lesson_status then there is no problem.
	 * However, the SCORM does not force the SCO to set the cmi.core.lesson_status.
	 * There is some additional requirements that must be adhered to successfully handle these cases:
	 * - Upon initial launch the LMS should set the cmi.core.lesson_status to "not attempted".
	 * - Upon receiving the LMSFinish() call or the user navigates away, the LMS should set the cmi.core.lesson_status for the SCO to "completed".
	 * - After setting the cmi.core.lesson_status to "completed", the LMS should now check to see if a Mastery Score has been specified
	 *   in the cmi.student_data.mastery_score, if supported, or the manifest that the SCO is a member of.
	 *   If a Mastery Score is provided and the SCO did set the cmi.core.score.raw, the LMS shall compare the cmi.core.score.raw
	 *   to the Mastery Score and set the cmi.core.lesson_status to either "passed" or "failed".
	 *   If no Mastery Score is provided, the LMS will leave the cmi.core.lesson_status as "completed"
	 * 
	 */		
	_setCmiCoreLessonStatus:function() {
		var lessonStatus = this._data.get('cmi.core.lesson_status', true);
		var lessonMode = this._data.get('cmi.core.lesson_mode', true);
		var masteryScore = this._data.get('cmi.student_data.mastery_score', true);
		var score = this._data.get('cmi.core.score.raw', true);
		
		if(lessonStatus == 'not attempted') {
			var value = lessonStatus;
			if(this._isCredit) {
				value = 'completed';
				if(masteryScore != '0') {
					if(score >= masteryScore) {
						value = 'passed';
					} else {
						value = 'failed';
					}
				}
			} else if(lessonMode == 'browse') {
				value = 'browsed';
			}
			var result = this._data.set('cmi.core.lesson_status', value, true);
			if(result != 'false') {
				this._storage.setItemToDataFinishObject('cmi.core.lesson_status', value);
			}
		}
	},
	
	/*
	 * _setItemToDataCommitObject
	 * 
	 * @param string parameter
	 * @param string value
	 */		
	_setItemToDataCommitObject:function(parameter, value) {
		if(this._isCredit) {
			this._storage.setItemToDataCommitObject(parameter, value);
		}
	},
	
	/*
	 * _setItemToDataFinishObject
	 * 
	 * @param string parameter
	 * @param string value
	 */		
	_setItemToDataFinishObject:function(parameter, value) {
		if(this._isCredit) {
			this._storage.setItemToDataFinishObject(parameter, value);
		}
	}
	
});