﻿/**
 * Data
 *
 * $Project: CLX.WBT-Tracker$
 * $Date: 12.07.2011 17:21:37$ 
 * $Revision: 5$
 *
 * @package    	clx.scorm
 * @copyright  	Copyright (c) Crealogix AG
 * @link		www.crealogix.com
 * @since       1.0.0
 * @author      Andreas Häberli <andreas.haeberli@crealogix.com>
 * @version		$Id: Data.js,v 1.2.1.0.1.0, 2011-07-12 15:21:37Z, Andreas Haeberli$
 */

dojo.require('clx.scorm.Datatype');	
dojo.require('clx.scorm.datatype.CMIBlank');
dojo.require('clx.scorm.datatype.CMIDecimal');
dojo.require('clx.scorm.datatype.CMIFeedback');
dojo.require('clx.scorm.datatype.CMIIdentifier');
dojo.require('clx.scorm.datatype.CMIInteger');
dojo.require('clx.scorm.datatype.CMIString');
dojo.require('clx.scorm.datatype.CMIString255');
dojo.require('clx.scorm.datatype.CMIString4096');
dojo.require('clx.scorm.datatype.CMIString65536');
dojo.require('clx.scorm.datatype.CMITime');
dojo.require('clx.scorm.datatype.CMITimespan');
dojo.require('clx.scorm.datatype.CMIVocabularyCredit');
dojo.require('clx.scorm.datatype.CMIVocabularyEntry');
dojo.require('clx.scorm.datatype.CMIVocabularyExit');
dojo.require('clx.scorm.datatype.CMIVocabularyInteraction');
dojo.require('clx.scorm.datatype.CMIVocabularyMode');
dojo.require('clx.scorm.datatype.CMICypVocabularyMode');
dojo.require('clx.scorm.datatype.CMIVocabularyResult');
dojo.require('clx.scorm.datatype.CMIVocabularyStatus');
dojo.require('clx.scorm.datatype.CMIVocabularyTimeLimitAction');

dojo.provide('clx.scorm.Data');
dojo.declare('clx.scorm.Data',null,{
	
	/*
	 * constructor
	 * 
	 * @param object storage
	 */		
	constructor: function(storage) {
		storage.setData(this);
		
		this._version = '1.2';
		this._items = {
			'cmi' : {
				'core' : {
					'_children' : '_children',
					'student_id' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIIdentifier()), true, false, ''),
					'student_name' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString255()), true, false, ''),
					'lesson_location' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString255()), true, true, ''),
					'credit' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyCredit()), true, false, 'credit'),
					'lesson_status' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyStatus()), true, true, 'not attempted'),
					'entry' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyEntry()), true, false, 'ab-imitio'),
					'score' : {
						'_children' : '_children',
						'raw' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIBlank(), new clx.scorm.datatype.CMIDecimal()), true, true, ''),
						'min' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIBlank(), new clx.scorm.datatype.CMIDecimal()), true, true, ''),
						'max' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIBlank(), new clx.scorm.datatype.CMIDecimal()), true, true, '')
					},
					'total_time' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMITimespan()), true, false, '0000:00:00'),
					'lesson_mode' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyMode()), true, false, 'normal'),
					'exit' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyExit()), false, true, ''),
					'session_time' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMITimespan()), false, true, '')
				},
				'suspend_data' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString()), true, true, ''),
				'launch_data' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString4096()), true, false, ''),
				'comments' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString4096()), true, true, ''),
				'comments_from_lms' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString4096()), true, false, ''),
				'objectives' : {
					'_children' : '_children',
					'_count' : '_count'
				},
				'student_data' : {
					'_children' : '_children',
					'mastery_score' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIDecimal()), true, false, '0'),
					'max_time_allowed' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMITimespan()), true, false, '0000:00:00'),
					'time_limit_action' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyTimeLimitAction()), false, true, 'exit,message')
				},
				'student_preference' : {
					'_children' : '_children',
					'audio' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIInteger()), true, true, '0'),
					'language' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString255()), true, true, ''),
					'speed' :  new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIInteger()), true, true, '0'),
					'text' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIInteger()), true, true, '0')
				},
				'interactions' : {
					'_children' : '_children',
					'_count' : '_count'
				}
			},

			//********************************			
			// proprietary additions for MyCYP
			//********************************
			'cyp' : {
				'lesson_mode' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMICypVocabularyMode()), true, false, 'Normal'),
				'variant' : {
					'id' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString4096()), false, true, ''),
					'version' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString4096()), false, true, '')
				},
				'counter' : {
					'starts' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIInteger()), true, false, '0'),
					'runs' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIInteger()), true, false, '0')
				},
				'last_run_timestamp' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString255()), true, true, ''), /* rinf 21.12.2018 Write-True */
			},
			
			// rinf neu
			
			/* rinf 21.12.2018 New items */
			'clx' : {
				'runtime_behavior' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString255()), true, false, ''),
				'lms_context': {
					'learningobjectid': new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIIdentifier()), true, false, ''),
					'moduleid': new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIString255()), true, false, '')
				}
			}

		};
		
		this._templates = {
			'cmi.objectives.n.id' : {
				'pattern' : new RegExp(/^cmi\.objectives\.[0-9]+\.id$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIIdentifier()), true, true, '')
			},
			'cmi.objectives.n.score.raw' : {
				'pattern' : new RegExp(/^cmi\.objectives\.[0-9]+\.score\.raw$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIBlank(), new clx.scorm.datatype.CMIDecimal()), true, true, '')
			},
	    	'cmi.objectives.n.score.max' : {
				'pattern' : new RegExp(/^cmi\.objectives\.[0-9]+\.score\.max$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIBlank(), new clx.scorm.datatype.CMIDecimal()), true, true, '')
			},
	    	'cmi.objectives.n.score.min' : {
				'pattern' : new RegExp(/^cmi\.objectives\.[0-9]+\.score\.min$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIBlank(), new clx.scorm.datatype.CMIDecimal()), true, true, '')
			},
	    	'cmi.objectives.n.status' : {
				'pattern' : new RegExp(/^cmi\.objectives\.[0-9]+\.status$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyStatus()), true, true, 'not attempted')
			},
	    	'cmi.interactions.n.id' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.id$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIIdentifier()), false, true, '')
			},
	    	'cmi.interactions.n.objectives.n.id' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.objectives\.[0-9]+\.id$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIIdentifier()), false, true, '')
			},
	    	'cmi.interactions.n.time' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.time$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMITime()), false, true, '00:00:00')
			},
	    	'cmi.interactions.n.type' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.type$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyInteraction()), false, true, 'true-false')
			},
	    	'cmi.interactions.n.correct_responses.n.pattern' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.correct_responses\.[0-9]+\.pattern$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIFeedback()), false, true, '')
			},
	    	'cmi.interactions.n.weighting' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.weighting$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIDecimal()), false, true, '0')
			},
	    	'cmi.interactions.n.student_response' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.student_response$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIFeedback()), false, true, '')
			},
	    	'cmi.interactions.n.result' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.result$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIVocabularyResult()), false, true, 'neutral')
			},
	    	'cmi.interactions.n.latency' : {
				'pattern' : new RegExp(/^cmi\.interactions\.[0-9]+\.latency$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMITimespan()), false, true, '0000:00:00')
			},

			//********************************			
			// proprietary additions for MyCYP
			//********************************
			'cyp.interactions.n.id' : {
				'pattern' : new RegExp(/^cyp\.interactions\.[0-9]+\.id$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIIdentifier()), false, true, '')
			},
			'cyp.interactions.n.response' : {
				'pattern' : new RegExp(/^cyp\.interactions\.[0-9]+\.response$/),
				'item' : new clx.scorm.Datatype(new Array(new clx.scorm.datatype.CMIFeedback()), false, true, '')
			}
		};
		
	},
	
	/*
	 * get
	 * 
	 * @param string parameter
	 * @param boolean withoutTypeCheck
	 * @return string
	 */		
	get:function(parameter, withoutTypeCheck) {
		var item = this._getItem(parameter);
		if(item) {
			switch (item) {
				case '_children':
					return this._getItemChildren(parameter);
				case '_count':
					return this._getItemCount(parameter);
				default:
					// Possibles Errors: 404
					return item.getValue(withoutTypeCheck);
				    break;
			}
		} else {
			var paramter_array = this._getParameterAsArray(parameter);
			switch (paramter_array[paramter_array.length-1]) {
				case '_version':
					return this._version;
				case '_children':
					return new clx.scorm.Error('202');
				case '_count':
					return new clx.scorm.Error('203');
				default:
				  // 401
				  return new clx.scorm.Error('201');
			  	  break;
			}
		}		
	},
	
	/*
	 * set
	 * 
	 * @param string parameter
	 * @param string value
	 * @param boolean withoutTypeCheck
	 * @return string
	 */		
	set:function(parameter, value, withoutTypeCheck) {
		var paramter_array = this._getParameterAsArray(parameter);
		switch (paramter_array[paramter_array.length-1]) {
			case '_version':
			case '_children':
			case '_count':
				return new clx.scorm.Error('402');
			default:
				var item = this._getItem(parameter);
				if(item) {
					if(typeof(item) == 'object') {
						value = item.filter(value);
						if(item.getValue(true) == value) {
							return 'false';
						} else {
							// Possibles Errors: 403, 405
							return item.setValue(value, withoutTypeCheck);
						}
					} else {
						return new clx.scorm.Error('401');
					}
				} else {
					return new clx.scorm.Error('201');
				}		    
		    	break;
		}
	},
	
	/*
	 * _getItem (private)
	 * 
	 * @param string parameter
	 * @return clx.scorm.Datatype
	 */		
	_getItem:function(parameter) {
		var paramter_array = this._getParameterAsArray(parameter);
		var item = this._items;
		for (var ii = 0; ii < paramter_array.length; ii++) {
			if(item[paramter_array[ii]] != null) {
				item = item[paramter_array[ii]];
			} else {
				item = this._createItemFromTemplate(parameter);
				if(item) {
					break;
				} else {
					return false;
				}
			}
		}
		return item;
	},
	
	/*
	 * _getParentItem (private)
	 * 
	 * @param string parameter
	 * @return clx.scorm.Datatype
	 */		
	_getParentItem:function(parameter) {
		var paramter_array = this._getParameterAsArray(parameter);
		var item = this._items;
		for (var ii = 0; ii < paramter_array.length-1; ii++) {
			item = item[paramter_array[ii]];			
		}
		return item;
	},
	
	/*
	 * _getParameterAsArray (private)
	 * 
	 * @param string parameter
	 * @return array
	 */		
	_getParameterAsArray:function(parameter) {
		return (parameter.toLowerCase()).split('.');
	},
	
	/*
	 * _getItemChildren (private)
	 * 
	 * @param string parameter
	 * @return string
	 */		
	_getItemChildren:function(parameter) {
		var item = this._getParentItem(parameter);
		var children_array = new Array();
		for (var key in item) {
			switch (key) {
				case '_count':
				case '_children':
					break;
				default:
					children_array.push(key);    
			    	break;
			}
		}
		return children_array.join(',');
	},
	
	/*
	 * _getItemCount (private)
	 * 
	 * @param string parameter
	 * @return string
	 */		
	_getItemCount:function(parameter) {
		var item = this._getParentItem(parameter);
		var count = 0;
		for (var key in item) {
			switch (key) {
				case '_count':
				case '_children':
					break;
				default:
					count++;
				break;
			}
		}
		return String(count);
	},
	
	/*
	 * _createItemFromTemplate (private)
	 * 
	 * @param string parameter
	 * @return mixed clx.scorm.Datatype|boolean
	 */		
	_createItemFromTemplate:function(parameter) {
		for (var key in this._templates) {
			if(this._templates[key]['pattern'].test(parameter)) {				
				var paramter_array = this._getParameterAsArray(parameter);				
				var item = this._items;				
				for (var ii = 0; ii < paramter_array.length; ii++) {
					if(item[paramter_array[ii]] == null) {
						if(ii == paramter_array.length-1) {
							item[paramter_array[ii]] = this._templates[key]['item'];
							return item[paramter_array[ii]];
						} else {
							item[paramter_array[ii]] = {};
						}
					}
					item = item[paramter_array[ii]];
				}
			}
		}
		return false;
	}
	
});