﻿/**
 * CMICypVocabularyMode
 * 
 * Special Mode Vocabulary for CYP.
 * Represents the the different modes that the CYP learnobject can
 * be launched in – other than the standard SCORM 1.2 modes “browse”, “normal” and “review”. 
 * 
 * Meaning as described in the Specification for CYP:
 * - Preview: 			Testpool/Autoren: Abspielen eines Tests zur Kontrolle
 *            			CYPnet/ContentAdmin: Abspielen des Tests in der AUflistung aller Lerninhalte
 * - PreviewQuestion: 	Testpool/Autoren: Abspiele einer einzelnen Frage zur Kontrolle
 * - Normal: 			CYPnet/Lernende: Durchführung der scharfen Prüfung
 * - Appeal 			CYPnet/Experte: Einsicht in die Prüfung im Beisein des Lernenden
 * - ResultsOnly: 		CYPnet/Lernenden: Anzeige der Testauswertung
 * 
 * $Project: CLX.CYPnet$
 * $Date: 21.12.2012$ 
 * $Revision: 1$
 *
 * @package    	clx.scorm.datatype
 * @copyright  	Copyright (c) Crealogix AG
 * @link		www.crealogix.com
 * @since       1.0.0
 * @author      Daniel Tiefenauer <daniel.tiefenauer@crealogix.com>
 * @version		$Id: CMICypVocabularyMode.js,v 1.0.0.0, 2012-12-21 Daniel Tiefenauer$
 */
 
dojo.provide('clx.scorm.datatype.CMICypVocabularyMode');
dojo.declare('clx.scorm.datatype.CMICypVocabularyMode',null,{
	
	constructor: function() {		
	},
	
	/*
	 * isValid
	 * 
	 * @param string value
	 * @return boolean
	 */
	isValid:function(value) {
		switch (value) {
			case 'Normal':
			case 'Preview':
			case 'PreviewQuestion':
			case 'Appeal':
			case 'ResultsOnly':
				return true;
			default:
				return false;
		} 	
	},

	/*
	 * Filter
	 * 
	 * @param string value
	 * @return string
	 */		
	filter:function(value) {
		return value; 	
	}
});