﻿/**
 * Air
 * 
 * $Project: CLX.WBT-Tracker$
 * $Date: 12.07.2011 17:21:37$ 
 * $Revision: 5$
 *
 * @package    	clx.scorm.storage
 * @copyright  	Copyright (c) Crealogix AG
 * @link		www.crealogix.com
 * @since       1.0.0
 * @version    	1.0.0.200901232230
 * @author      Andreas Häberli <andreas.haeberli@crealogix.com>
 */

dojo.require("clx.scorm.storage.Abstract");
dojo.require("clx.scorm.addon.cyp.air.bridge.Command");

dojo.provide("clx.scorm.storage.Air");
dojo.declare("clx.scorm.storage.Air", clx.scorm.storage.Abstract,{
	
	/*
	 * constructor
	 * 
	 * @param string session_id
	 */		
	constructor: function(session_id, bridge) {
		this.inherited("constructor", arguments);
		
		this._session_id = session_id;
		this._bridge = bridge;
		
		this._version = '1.0';
		this._isSynch = false;		
	},
	
	/*
	 * getSessionId
	 * 
	 * @return string
	 */		
	getSessionId:function() {
		return this._session_id;
	},
	
	/*
	 * read
	 * Initialize the data session and read data from backend
	 * 
	 * @return boolean
	 */		
	read:function() {
		var request_obj = new Object({
			'command'		: 'Read',
			'session_id' 	: this._session_id,
			'version' 		: this._version
		});
		this._bridge.call(new clx.scorm.addon.cyp.air.bridge.Command(this, 'communicate', 'onAfterRead', encodeURIComponent(dojo.toJson(request_obj))));
		return true;
	},
	
	/*
	 * Callback on after read
	 * 
	 * @return void
	 */		
	onAfterRead:function(response) {
		var response_obj = dojo.fromJson(response);
		if(response_obj['error'] == '0') {
			for (var key in response_obj['data']) {
				var result = this._data.set(key, response_obj['data'][key], true);
				if(typeof(result) == 'object') {
					break;
				}
			}
		}	
	},
	
	/*
	 * commit
	 * Send data to backend
	 * 
	 * @return boolean
	 */		
	commit:function() {
		var data_obj = this._getDataCommitObject();
		if(data_obj) {
			var request_obj = new Object({
				'command'		: 'Commit',
				'session_id' 	: this._session_id,
				'version' 		: this._version,
				'data'			: data_obj
			});
			this._bridge.call(new clx.scorm.addon.cyp.air.bridge.Command(this, 'communicate', 'onAfterCommit', encodeURIComponent(dojo.toJson(request_obj))));
		}
		return true;
	},	
	
	/*
	 * Callback on after commit
	 * 
	 * @return void
	 */	
	onAfterCommit:function(response) {
		var response_obj = dojo.fromJson(response);
		if(typeof(response_obj) == 'object') {
			if(response_obj['error'] == '0') {
				this._clearDataCommitObject();
			}
		}
	},
	
	/*
	 * finish
	 * Finish the data session
	 * 
	 * @return boolean
	 */		
	finish:function() {
		var request_obj = new Object({
			'command'		: 'Finish',
			'session_id' 	: this._session_id,
			'version' 		: this._version,
			'data'			: this._getDataFinishObject()		
		});	
		this._bridge.call(new clx.scorm.addon.cyp.air.bridge.Command(this, 'communicate', 'onAfterFinish', encodeURIComponent(dojo.toJson(request_obj))));
		return true;
	},
	
	/*
	 * Callback on after finish
	 * 
	 * @return void
	 */		
	onAfterFinish:function(response) {
		var response_obj = dojo.fromJson(response);
		if(response_obj['error'] == '0') {
		}
	},
		
	/*
	 * toInfo
	 * 
	 */		
	toInfo:function() {
		return 'Storage AIR (Session ID: '+this.getSessionId()+')';
	}
});