﻿/**
 * Json
 * 
 * $Project: CLX.WBT-Tracker$
 * $Date: 12.07.2011 17:21:37$ 
 * $Revision: 5$
 *
 * @package    	clx.scorm.storage
 * @copyright  	Copyright (c) Crealogix AG
 * @link		www.crealogix.com
 * @since       1.0.0
 * @version    	1.0.0.200901232230
 * @author      Andreas Häberli <andreas.haeberli@crealogix.com>
 */

dojo.require("clx.scorm.storage.Abstract");

dojo.provide("clx.scorm.storage.Json");
dojo.declare("clx.scorm.storage.Json", clx.scorm.storage.Abstract,{
	
	/*
	 * constructor
	 * 
	 * @param string session_id
	 * @param string url
	 */		
	constructor: function(session_id, url) {
		this.inherited("constructor", arguments);
		
		this._session_id = session_id;
		this._url = url;
		this._response_obj = null;		
		
		this._version = '1.2';
		this._timeout = 5000;
		this._isSynch = true;
	},
	
	/*
	 * getSessionId
	 * 
	 * @return string
	 */		
	getSessionId:function() {
		return this._session_id;
	},
	
	/*
	 * getUrl
	 * 
	 * @return string
	 */		
	getUrl:function() {
		return this._url;
	},
	
	/*
	 * read
	 * Initialize the data session and read data from backend
	 * 
	 * @return boolean
	 */		
	read:function() {
		var request_obj = new Object({
			'ref_obj' 	: this,
			'sync'		: true,
			'handleAs' 	: 'json',
			'timeout' 	: this._timeout,
			'url' 		: this._url,
			'load' 		: this._onLoad,
			'error' 	: this._onError,
			'postData' 	: dojo.toJson({
				'command'		: 'Read',
				'session_id' 	: this._session_id,
				'version' 		: this._version
			})
		});
		// console.log('<--- '+request_obj['postData']);
		dojo.rawXhrPost(request_obj);
		
		// after onLoad
		if(typeof(this._response_obj) == 'object') {
			for (var key in this._response_obj) {
				// console.log('---> '+key+' -> '+this._response_obj[key]);
			}
			if(this._response_obj['error'] == '0') {
				for (var key in this._response_obj['data']) {
					var result = this._data.set(key, this._response_obj['data'][key], true);
					// console.log('-----> '+key+' -> '+this._response_obj['data'][key]+' ('+!(typeof(result) == 'object')+')');
					if(typeof(result) == 'object') {
						return false;
					}
				}
				return true;
			}
		}
		return false;
	},
	
	/*
	 * commit
	 * Send data to backend
	 * 
	 * @return boolean
	 */		
	commit:function() {
		var data_obj = this._getDataCommitObject();
		if(data_obj) {
			var request_obj = new Object({
				'ref_obj' 	: this,
				'sync'		: true,
				'handleAs' 	: 'json',
				'timeout' 	: this._timeout,
				'url' 		: this._url,
				'load' 		: this._onLoad,
				'error' 	: this._onError,
				'postData' 	: dojo.toJson({
					'command'		: 'Commit',
					'session_id' 	: this._session_id,
					'version' 		: this._version,
					'data'			: encodeURIComponent(dojo.toJson(data_obj))
				})
			});
			// console.log('<--- '+request_obj['postData']);
			dojo.rawXhrPost(request_obj);

			// after onLoad
			if(typeof(this._response_obj) == 'object') {
				for (var key in this._response_obj) {
					// console.log('---> '+key+' -> '+this._response_obj[key]);
				}
				if(this._response_obj['error'] == '0') {
					this._clearDataCommitObject();
					return true;
				}
			}
			return false;
		} else {
			// console.log('---- no data to commit');
			return true;
		}	 		
	},
	
	/*
	 * finish
	 * Finish the data session
	 * 
	 * @return boolean
	 */		
	finish:function() {
		var request_obj = new Object({
			'ref_obj' 	: this,
			'sync'		: true,
			'handleAs' 	: 'json',
			'timeout' 	: this._timeout,
			'url' 		: this._url,
			'load' 		: this._onLoad,
			'error' 	: this._onError,
			'postData' 	: dojo.toJson({
				'command'		: 'Finish',
				'session_id' 	: this._session_id,
				'version' 		: this._version,
				'data'			: encodeURIComponent(dojo.toJson(this._getDataFinishObject()))
			})
		});
		
		// console.log('<--- '+request_obj['postData']);
		dojo.rawXhrPost(request_obj);
	
		// after onLoad
		if(typeof(this._response_obj) == 'object') {
			for (var key in this._response_obj) {
				// console.log('---> '+key+' -> '+this._response_obj[key]);
			}
			if(this._response_obj['error'] == '0') {
				return true;
			}
		}
		return false;
	},
	
	/*
	 * _onLoad (private)
	 * 
	 * @param object response_obj
	 */		
	_onLoad:function(response_obj) {
		this.ref_obj._response_obj = response_obj;
	},
	
	/*
	 * _onError (private)
	 * 
	 * @param object response_obj
	 * @param object ioArgs
	 */		
	_onError: function(response_obj, ioArgs) {
		// console.error("HTTP status code: ", ioArgs.xhr.status);
		this.ref_obj._response_obj = null;
	},
	
	/*
	 * toInfo
	 * 
	 */		
	toInfo:function() {
		return 'Storage JSON (Session ID: '+this.getSessionId() + ' / ' + 'URL: '+this.getUrl()+')';
	}	
});